"""PyPi package with sort func."""

from .functions import methods
from pysorters.functions.methods import Sorter, gran_array_int
from pysorters.functions.methods import Sorter, gran_array_int, gran_array_float
