"""
This module contains the automations for the pysorters package.
"""

from pysorters.automations import filepy
from pysorters.automations.filepy import Pyxplorer
from pysorters.automations import sweepy
from pysorters.automations.sweepy import Sweepy
import pysorters.automations.sweepy
import pysorters.automations.filepy
